@@ -120,6 +120,17 @@ typedef struct {
 } Gt_OutputData;
 
 
+/*****
+ * helper
+ **/
+
+static inline int
+constrain(int low, int x, int high)
+{
+  return x < low ? low : (x < high ? x : high);
+}
+
+
 /*****
  * error & verbose
  **/
@@ -189,7 +200,7 @@ Gt_ColorTransform *append_color_change
 
 void	pipe_color_transformer(Gif_Colormap *, void *);
 
-int	crop_image(Gif_Image *, Gt_Crop *, int first_image);
+int	crop_image(Gif_Image *, Gt_Crop *, int preserve_total_crop);
 
 void	flip_image(Gif_Image *, int scr_width, int scr_height, int is_vert);
 void	rotate_image(Gif_Image *, int scr_width, int scr_height, int rotation);
